@@ -1280,10 +1280,7 @@ class TimestampCol(Col):
     baseClass = SOTimestampCol
 
 
-try:
-    from decimal import Decimal
-except ImportError:
-    Decimal = float
+from decimal import Decimal
 
 class DecimalValidator(validators.Validator):
     def to_python(self, value, state):
@@ -1353,6 +1350,54 @@ class CurrencyCol(DecimalCol):
     baseClass = SOCurrencyCol
 
 
+class DecimalStringValidator(DecimalValidator):
+    def to_python(self, value, state):
+        value = super(DecimalStringValidator, self).to_python(value, state)
+        if self.precision and isinstance(value, Decimal):
+            assert value < self.max, \
+                    "Value must be less than %s" % int(self.max)
+            value = value.quantize(self.precision)
+        return value
+
+    def from_python(self, value, state):
+        value = super(DecimalStringValidator, self).from_python(value, state)
+        if isinstance(value, Decimal):
+            if self.precision:
+                assert value < self.max, \
+                        "Value must be less than %s" % int(self.max)
+                value = value.quantize(self.precision)
+            value = value.to_eng_string()
+        elif isinstance(value, (int, long)):
+            value = str(value)
+        return value
+
+class SODecimalStringCol(SOStringCol):
+    def __init__(self, **kw):
+        self.size = kw.pop('size', NoDefault)
+        assert (self.size is not NoDefault) and (self.size >= 0), \
+            "You must give a size argument as a positive integer"
+        self.precision = kw.pop('precision', NoDefault)
+        assert (self.precision is not NoDefault) and (self.precision >= 0), \
+               "You must give a precision argument as a positive integer"
+        kw['length'] = int(self.size) + int(self.precision)
+        self.quantize = kw.pop('quantize', False)
+        assert isinstance(self.quantize, bool), \
+                "quantize argument must be Boolean True/False"
+        super(SODecimalStringCol, self).__init__(**kw)
+
+    def createValidators(self):
+        if self.quantize:
+            v = DecimalStringValidator(
+                precision=Decimal(10) ** (-1 * int(self.precision)),
+                max=Decimal(10) ** (int(self.size) - int(self.precision)))
+        else:
+            v = DecimalStringValidator(precision=0)
+        return [v] + super(SODecimalStringCol, self).createValidators()
+
+class DecimalStringCol(StringCol):
+    baseClass = SODecimalStringCol
+
+
 class BinaryValidator(validators.Validator):
     """
     Validator for binary types.
