@@ -13,7 +13,7 @@ class PostgresConnection(DBAPI):
     schemes = [dbName, 'postgresql', 'psycopg']
 
     def __init__(self, dsn=None, host=None, port=None, db=None,
-                 user=None, passwd=None, usePygresql=False, unicodeCols=False,
+                 user=None, password=None, usePygresql=False, unicodeCols=False,
                  **kw):
         global psycopg, pgdb
         self.usePygresql = usePygresql
@@ -23,7 +23,10 @@ class PostgresConnection(DBAPI):
             self.module = pgdb
         else:
             if psycopg is None:
-                import psycopg
+                try:
+                    import psycopg2 as psycopg
+                except ImportError:
+                    import psycopg
             self.module = psycopg
 
             # Register a converter for psycopg Binary type.
@@ -34,7 +37,7 @@ class PostgresConnection(DBAPI):
         self.host = host
         self.port = port
         self.db = db
-        self.password = passwd
+        self.password = password
         self.dsn_dict = dsn_dict = {}
         if host:
             dsn_dict["host"] = host
@@ -42,13 +45,16 @@ class PostgresConnection(DBAPI):
             if usePygresql:
                 dsn_dict["host"] = "%s:%d" % (host, port)
             else:
-                dsn_dict["port"] = str(port)
+                if psycopg.__version__.split('.')[0] == '1':
+                    dsn_dict["port"] = str(port)
+                else:
+                    dsn_dict["port"] = port
         if db:
             dsn_dict["database"] = db
         if user:
             dsn_dict["user"] = user
-        if passwd:
-            dsn_dict["password"] = passwd
+        if password:
+            dsn_dict["password"] = password
         self.use_dsn = dsn is not None
         if dsn is None:
             if usePygresql:
@@ -62,16 +68,16 @@ class PostgresConnection(DBAPI):
                 if user:
                     dsn += user
                 dsn += ':'
-                if passwd:
-                    dsn += passwd
+                if password:
+                    dsn += password
             else:
                 dsn = []
                 if db:
                     dsn.append('dbname=%s' % db)
                 if user:
                     dsn.append('user=%s' % user)
-                if passwd:
-                    dsn.append('password=%s' % passwd)
+                if password:
+                    dsn.append('password=%s' % password)
                 if host:
                     dsn.append('host=%s' % host)
                 if port:
@@ -79,15 +85,17 @@ class PostgresConnection(DBAPI):
                 dsn = ' '.join(dsn)
         self.dsn = dsn
         self.unicodeCols = unicodeCols
+        self.schema = kw.pop('schema', None)
         DBAPI.__init__(self, **kw)
 
-        # Server version cache
-        self._server_version = None # Not yet initialized
-
     def connectionFromURI(cls, uri):
         user, password, host, port, path, args = cls._parseURI(uri)
         path = path.strip('/')
-        return cls(host=host, port=port, db=path, user=user, passwd=password, **args)
+        if (host is None) and path.count('/'): # Non-default unix socket
+            path_parts = path.split('/')
+            host = '/' + '/'.join(path_parts[:-1])
+            path = path_parts[-1]
+        return cls(host=host, port=port, db=path, user=user, password=password, **args)
     connectionFromURI = classmethod(connectionFromURI)
 
     def _setAutoCommit(self, conn, auto):
@@ -107,13 +115,16 @@ class PostgresConnection(DBAPI):
             # psycopg2 does not have an autocommit method.
             if hasattr(conn, 'autocommit'):
                 conn.autocommit(1)
+        if self.schema:
+            c = conn.cursor()
+            c.execute("SET search_path TO " + self.schema)
         return conn
 
     def _queryInsertID(self, conn, soInstance, id, names, values):
         table = soInstance.sqlmeta.table
         idName = soInstance.sqlmeta.idName
-        sequenceName = getattr(soInstance, '_idSequence',
-                               '%s_%s_seq' % (table, idName))
+        sequenceName = soInstance.sqlmeta.idSequence or \
+                               '%s_%s_seq' % (table, idName)
         c = conn.cursor()
         if id is None:
             c.execute("SELECT NEXTVAL('%s')" % sequenceName)
@@ -128,25 +139,28 @@ class PostgresConnection(DBAPI):
             self.printDebug(conn, id, 'QueryIns', 'result')
         return id
 
-    def _queryAddLimitOffset(self, query, start, end):
+    def _queryAddLimitOffset(cls, query, start, end):
         if not start:
             return "%s LIMIT %i" % (query, end)
         if not end:
             return "%s OFFSET %i" % (query, start)
         return "%s LIMIT %i OFFSET %i" % (query, end-start, start)
+    _queryAddLimitOffset = classmethod(_queryAddLimitOffset)
 
     def createColumn(self, soClass, col):
         return col.postgresCreateSQL()
 
+    def createReferenceConstraint(self, soClass, col):
+        return col.postgresCreateReferenceConstraint()
+
     def createIndexSQL(self, soClass, index):
         return index.postgresCreateIndexSQL(soClass)
 
     def createIDColumn(self, soClass):
-        return '%s SERIAL PRIMARY KEY' % soClass.sqlmeta.idName
+        key_type = {int: "SERIAL", str: "TEXT"}[soClass.sqlmeta.idType]
+        return '%s %s PRIMARY KEY' % (soClass.sqlmeta.idName, key_type)
 
     def dropTable(self, tableName, cascade=False):
-        if self.server_version[:3] <= "7.2":
-            cascade=False
         self.query("DROP TABLE %s %s" % (tableName,
                                          cascade and 'CASCADE' or ''))
 
@@ -163,10 +177,8 @@ class PostgresConnection(DBAPI):
                    (tableName,
                     column.postgresCreateSQL()))
 
-    def delColumn(self, tableName, column):
-        self.query('ALTER TABLE %s DROP COLUMN %s' %
-                   (tableName,
-                    column.dbName))
+    def delColumn(self, sqlmeta, column):
+        self.query('ALTER TABLE %s DROP COLUMN %s' % (sqlmeta.table, column.dbName))
 
     def columnsFromSchema(self, tableName, soClass):
 
@@ -226,26 +238,34 @@ class PostgresConnection(DBAPI):
         for field, t, notnull, defaultstr in colData:
             if field == primaryKey:
                 continue
-            colClass, kw = self.guessClass(t)
-            if self.unicodeCols and colClass == col.StringCol:
-                colClass = col.UnicodeCol
-                kw['dbEncoding'] = client_encoding
-            kw['name'] = soClass.sqlmeta.style.dbColumnToPythonAttr(field)
+            if keymap.has_key(field):
+                colClass = col.ForeignKey
+                kw = {'foreignKey': soClass.sqlmeta.style.dbTableToPythonClass(keymap[field])}
+                name = soClass.sqlmeta.style.dbColumnToPythonAttr(field)
+                if name.endswith('ID'):
+                    name = name[:-2]
+                kw['name'] = name
+            else:
+                colClass, kw = self.guessClass(t)
+                if self.unicodeCols and colClass is col.StringCol:
+                    colClass = col.UnicodeCol
+                    kw['dbEncoding'] = client_encoding
+                kw['name'] = soClass.sqlmeta.style.dbColumnToPythonAttr(field)
             kw['dbName'] = field
             kw['notNone'] = notnull
             if defaultstr is not None:
                 kw['default'] = self.defaultFromSchema(colClass, defaultstr)
             elif not notnull:
                 kw['default'] = None
-            if keymap.has_key(field):
-                kw['foreignKey'] = keymap[field]
             results.append(colClass(**kw))
         return results
 
     def guessClass(self, t):
-        if t.count('int'):
+        if t.count('point'): # poINT before INT
+            return col.StringCol, {}
+        elif t.count('int'):
             return col.IntCol, {}
-        elif t.count('varying'):
+        elif t.count('varying') or t.count('varchar'):
             if '(' in t:
                 return col.StringCol, {'length': int(t[t.index('(')+1:-1])}
             else: # varchar without length in Postgres means any length
@@ -253,10 +273,16 @@ class PostgresConnection(DBAPI):
         elif t.startswith('character('):
             return col.StringCol, {'length': int(t[t.index('(')+1:-1]),
                                    'varchar': False}
+        elif t.count('float') or t.count('real') or t.count('double'):
+            return col.FloatCol, {}
         elif t == 'text':
             return col.StringCol, {}
+        elif t.startswith('timestamp'):
+            return col.DateTimeCol, {}
         elif t.startswith('datetime'):
             return col.DateTimeCol, {}
+        elif t.startswith('date'):
+            return col.DateCol, {}
         elif t.startswith('bool'):
             return col.BoolCol, {}
         elif t.startswith('bytea'):
@@ -276,16 +302,7 @@ class PostgresConnection(DBAPI):
                 return True
         return getattr(sqlbuilder.const, defaultstr)
 
-    def server_version(self):
-        if self._server_version is None:
-            # The result is something like
-            # ' PostgreSQL 7.2.1 on i686-pc-linux-gnu, compiled by GCC 2.95.4'
-            server_version = self.queryOne("SELECT version()")[0]
-            self._server_version = server_version.split()[1]
-        return self._server_version
-    server_version = property(server_version)
-
-    def createEmptyDatabase(self):
+    def _createOrDropDatabase(self, op="CREATE"):
         # We have to connect to *some* database, so we'll connect to
         # template1, which is a common open database.
         # @@: This doesn't use self.use_dsn or self.dsn_dict
@@ -305,13 +322,15 @@ class PostgresConnection(DBAPI):
         # We must close the transaction with a commit so that
         # the CREATE DATABASE can work (which can't be in a transaction):
         cur.execute('COMMIT')
-        # And we can't use template1 since we're connected to template1,
-        # so we use template0.  @@: What's the difference between
-        # these two templates?
-        cur.execute('CREATE DATABASE %s TEMPLATE=template0' % self.db)
+        cur.execute('%s DATABASE %s' % (op, self.db))
         cur.close()
         conn.close()
 
+    def createEmptyDatabase(self):
+        self._createOrDropDatabase()
+
+    def dropDatabase(self):
+        self._createOrDropDatabase(op="DROP")
 
 
 # Converter for psycopg Binary type.
