@@ -1,11 +1,9 @@
+from datetime import datetime
+now = datetime.now
+
 from sqlobject import *
 from sqlobject.tests.dbtest import *
 from sqlobject import classregistry
-try:
-    from datetime import datetime
-    now = datetime.now
-except ImportError:
-    from mx.DateTime import now
 
 ########################################
 ## Auto class generation
